@@ -1,82 +1,88 @@
 """A collection of particle info containers.
 
-The `.particle` module is the starting point of `qrules`. Its main interface is
-the `ParticleCollection`, which is a collection of immutable `Particle`
-instances that are uniquely defined by their properties. As such, it can be
-used stand-alone as a database of quantum numbers (see :doc:`/usage/particle`).
+The :mod:`.particle` module is the starting point of `qrules`. Its main interface is the
+`ParticleCollection`, which is a collection of immutable `Particle` instances that are
+uniquely defined by their properties. As such, it can be used stand-alone as a database
+of quantum numbers (see :doc:`/usage/particle`).
 
-The `.transition` module uses the properties of `Particle` instances when it
-computes which `.StateTransitionGraph` s are allowed between an initial state
-and final state.
+The `.transition` module uses the properties of `Particle` instances when it computes
+which `.MutableTransition` s are allowed between an initial state and final state.
 """
 
+from __future__ import annotations
+
 import logging
 import re
+import sys
 from collections import abc
 from difflib import get_close_matches
+from fractions import Fraction
+from functools import total_ordering
 from math import copysign
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Iterable,
-    Iterator,
-    Optional,
-    Set,
-    SupportsFloat,
-    Tuple,
-    Union,
-)
+from typing import TYPE_CHECKING, Any
 
-import attr
-from attr.converters import optional
-from attr.validators import instance_of
-from particle import Particle as PdgDatabase
-from particle.particle import enums
+import attrs
+from attrs import field, frozen
+from attrs.converters import optional
+from attrs.validators import instance_of
 
-from .conservation_rules import GellMannNishijimaInput, gellmann_nishijima
-from .quantum_numbers import Parity, _to_fraction
+from qrules._attrs import to_fraction, to_parity
+from qrules.conservation_rules import GellMannNishijimaInput, gellmann_nishijima
+from qrules.quantum_numbers import Parity, _float_as_signed_str
 
-try:
+if sys.version_info >= (3, 11):
+    from typing import Self
+else:
+    from typing_extensions import Self
+if TYPE_CHECKING:
+    from collections.abc import Callable, Iterable, Iterator
+
+    from attrs import Attribute
     from IPython.lib.pretty import PrettyPrinter
-except ImportError:
-    PrettyPrinter = Any
+    from particle import Particle as PdgDatabase
+    from particle.particle import enums
+
+_LOGGER = logging.getLogger(__name__)
 
 
-def _to_float(value: SupportsFloat) -> float:
-    float_value = float(value)
-    if float_value == -0.0:
-        float_value = 0.0
-    return float_value
+def _validate_fraction_for_spin(
+    instance: Spin,
+    attribute: Attribute,  # noqa: ARG001
+    value: Fraction,  # noqa: ARG001
+) -> Any:
+    if instance.magnitude % Fraction(1, 2) != Fraction(0):
+        msg = f"Spin magnitude {instance.magnitude} has to be a multitude of 0.5"
+        raise ValueError(msg)
+    if abs(instance.projection) > instance.magnitude:
+        if instance.magnitude < Fraction(0):
+            msg = f"Spin magnitude has to be positive, but is {instance.magnitude}"
+            raise ValueError(msg)
+        msg = (
+            "Absolute value of spin projection cannot be larger than its"
+            f" magnitude:\n abs({instance.projection}) > {instance.magnitude}"
+        )
+        raise ValueError(msg)
+    if (instance.projection - instance.magnitude).denominator != 1:
+        msg = (
+            f"{type(instance).__name__}{(instance.magnitude, instance.projection)}: (projection -"
+            " magnitude) should be integer"
+        )
+        raise ValueError(msg)
 
 
-@attr.s(frozen=True, eq=False, hash=True)
-class Spin:
+@total_ordering
+@frozen(eq=False, hash=True, order=False)
+class Spin:  # noqa: PLW1641
     """Safe, immutable data container for spin **with projection**."""
 
-    magnitude: float = attr.ib(converter=_to_float)
-    projection: float = attr.ib(converter=_to_float)
-
-    def __attrs_post_init__(self) -> None:
-        if self.magnitude % 0.5 != 0.0:
-            raise ValueError(
-                f"Spin magnitude {self.magnitude} has to be a multitude of 0.5"
-            )
-        if abs(self.projection) > self.magnitude:
-            if self.magnitude < 0.0:
-                raise ValueError(
-                    "Spin magnitude has to be positive:\n" f" {self.magnitude}"
-                )
-            raise ValueError(
-                "Absolute value of spin projection cannot be larger than its "
-                "magnitude:\n"
-                f" abs({self.projection}) > {self.magnitude}"
-            )
-        if not (self.projection - self.magnitude).is_integer():
-            raise ValueError(
-                f"{self.__class__.__name__}{(self.magnitude, self.projection)}: "
-                "(projection - magnitude) should be integer! "
-            )
+    magnitude: Fraction = field(
+        converter=to_fraction,
+        validator=_validate_fraction_for_spin,
+    )
+    projection: Fraction = field(
+        converter=to_fraction,
+        validator=_validate_fraction_for_spin,
+    )
 
     def __eq__(self, other: object) -> bool:
         if isinstance(other, Spin):
@@ -87,45 +93,52 @@ class Spin:
         return self.magnitude == other
 
     def __float__(self) -> float:
-        return self.magnitude
+        return float(self.magnitude)
 
-    def __neg__(self) -> "Spin":
+    def __gt__(self, other: Any) -> bool:
+        if isinstance(other, Spin):
+            return attrs.astuple(self) > attrs.astuple(other)
+        return self.magnitude > other
+
+    def __neg__(self) -> Spin:
         return Spin(self.magnitude, -self.projection)
 
     def __repr__(self) -> str:
-        return f"{self.__class__.__name__}{(self.magnitude, self.projection)}"
+        return f"{type(self).__name__}{(self.magnitude, self.projection)}"
 
     def _repr_pretty_(self, p: PrettyPrinter, _: bool) -> None:
         class_name = type(self).__name__
-        magnitude = _to_fraction(self.magnitude)
-        projection = _to_fraction(self.projection, render_plus=True)
+        magnitude = _render_fraction(self.magnitude)
+        projection = _render_fraction(self.projection, plusminus=True)
         p.text(f"{class_name}({magnitude}, {projection})")
 
 
-def _to_parity(value: Union[Parity, int]) -> Parity:
-    return Parity(int(value))
+def _render_fraction(fraction: Fraction, plusminus: bool = False) -> str:
+    if plusminus and fraction.numerator > 0:
+        return f"+{fraction}"
+    return str(fraction)
 
 
-def _to_spin(value: Union[Spin, Tuple[float, float]]) -> Spin:
+def _to_spin(value: Spin | tuple[Fraction, Fraction] | tuple[float, float]) -> Spin:
     if isinstance(value, tuple):
         return Spin(*value)
     return value
 
 
-@attr.s(frozen=True, repr=True, kw_only=True)
-class Particle:  # pylint: disable=too-many-instance-attributes
+@total_ordering
+@frozen(kw_only=True, order=False, repr=True)
+class Particle:
     """Immutable container of data defining a physical particle.
 
-    A `Particle` is defined by the minimum set of the quantum numbers that
-    every possible instances of that particle have in common (the "static"
-    quantum numbers of the particle). A "non-static" quantum number is the spin
-    projection. Hence `Particle` instances do **not** contain spin projection
-    information.
+    A `Particle` is defined by the minimum set of the quantum numbers that every
+    possible instances of that particle have in common (the "static" quantum numbers of
+    the particle). A "non-static" quantum number is the spin projection. Hence
+    `Particle` instances do **not** contain spin projection information.
 
-    `Particle` instances are uniquely defined by their quantum numbers and
-    properties like `~Particle.mass`. The `~Particle.name` and `~Particle.pid`
-    are therefore just labels that are not taken into account when checking if
-    two `Particle` instances are equal.
+    `Particle` instances are uniquely defined by their quantum numbers and properties
+    like `~Particle.mass`. The `~Particle.name` and `~Particle.pid` are therefore just
+    labels that are not taken into account when checking if two `Particle` instances are
+    equal.
 
     .. note:: As opposed to classes such as `.EdgeQuantumNumbers` and
         `.NodeQuantumNumbers`, the `Particle` class serves as an interface to
@@ -133,44 +146,32 @@ class Particle:  # pylint: disable=too-many-instance-attributes
     """
 
     # Labels
-    name: str = attr.ib(eq=False)
-    pid: int = attr.ib(eq=False)
-    latex: Optional[str] = attr.ib(eq=False, default=None)
+    name: str = field(eq=False)
+    pid: int = field(eq=False)
+    latex: str | None = field(eq=False, default=None)
     # Unique properties
-    spin: float = attr.ib(converter=float)
-    mass: float = attr.ib(converter=float)
-    width: float = attr.ib(converter=float, default=0.0)
-    charge: int = attr.ib(default=0)
-    isospin: Optional[Spin] = attr.ib(
-        converter=optional(_to_spin), default=None
-    )
-    strangeness: int = attr.ib(default=0, validator=instance_of(int))
-    charmness: int = attr.ib(default=0, validator=instance_of(int))
-    bottomness: int = attr.ib(default=0, validator=instance_of(int))
-    topness: int = attr.ib(default=0, validator=instance_of(int))
-    baryon_number: int = attr.ib(default=0, validator=instance_of(int))
-    electron_lepton_number: int = attr.ib(
-        default=0, validator=instance_of(int)
-    )
-    muon_lepton_number: int = attr.ib(default=0, validator=instance_of(int))
-    tau_lepton_number: int = attr.ib(default=0, validator=instance_of(int))
-    parity: Optional[Parity] = attr.ib(
-        converter=optional(_to_parity), default=None
-    )
-    c_parity: Optional[Parity] = attr.ib(
-        converter=optional(_to_parity), default=None
-    )
-    g_parity: Optional[Parity] = attr.ib(
-        converter=optional(_to_parity), default=None
-    )
+    spin: Fraction = field(converter=Fraction)
+    mass: float = field(converter=float)
+    width: float = field(converter=float, default=0.0)
+    charge: int = field(default=0)
+    isospin: Spin | None = field(converter=optional(_to_spin), default=None)
+    strangeness: int = field(default=0, validator=instance_of(int))
+    charmness: int = field(default=0, validator=instance_of(int))
+    bottomness: int = field(default=0, validator=instance_of(int))
+    topness: int = field(default=0, validator=instance_of(int))
+    baryon_number: int = field(default=0, validator=instance_of(int))
+    electron_lepton_number: int = field(default=0, validator=instance_of(int))
+    muon_lepton_number: int = field(default=0, validator=instance_of(int))
+    tau_lepton_number: int = field(default=0, validator=instance_of(int))
+    parity: Parity | None = field(converter=optional(to_parity), default=None)
+    c_parity: Parity | None = field(converter=optional(to_parity), default=None)
+    g_parity: Parity | None = field(converter=optional(to_parity), default=None)
 
     def __attrs_post_init__(self) -> None:
         if self.isospin is not None and not gellmann_nishijima(
             GellMannNishijimaInput(
                 charge=self.charge,
-                isospin_projection=self.isospin.projection
-                if self.isospin
-                else None,
+                isospin_projection=self.isospin.projection if self.isospin else None,
                 strangeness=self.strangeness,
                 charmness=self.charmness,
                 bottomness=self.bottomness,
@@ -181,20 +182,33 @@ class Particle:  # pylint: disable=too-many-instance-attributes
                 tau_lepton_number=self.tau_lepton_number,
             )
         ):
-            raise ValueError(
-                f"Cannot construct particle {self.name}, because its quantum"
-                " numbers don't agree with the Gell-Mann–Nishijima formula:\n"
-                f"  Q[{self.charge}] != "
-                f"Iz[{self.isospin.projection if self.isospin else 0}] + 1/2 "
-                f"(B[{self.baryon_number}] + "
-                f" S[{self.strangeness}] + "
-                f" C[{self.charmness}] +"
-                f" B'[{self.bottomness}] +"
-                f" T[{self.strangeness}]"
-                ")"
+            msg = (
+                f"Cannot construct particle {self.name}, because its quantum numbers"
+                " don't agree with the Gell-Mann-Nishijima formula:\n "
+                f" Q[{self.charge}] !="
+                f" Iz[{self.isospin.projection if self.isospin else 0}] + 1/2"
+                f" (B[{self.baryon_number}] +  S[{self.strangeness}] + "
+                f" C[{self.charmness}] + B'[{self.bottomness}] + T[{self.topness}])"
             )
+            raise ValueError(msg)
 
-    def __neg__(self) -> "Particle":
+    def __gt__(self, other: Any) -> bool:
+        if isinstance(other, Particle):
+
+            def sorting_key(particle: Particle) -> tuple:
+                name_root = _get_name_root(particle.name)
+                return (
+                    name_root[0].lower(),
+                    name_root,
+                    particle.mass,
+                    particle.charge,
+                )
+
+            return sorting_key(self) > sorting_key(other)
+        msg = f"Cannot compare {type(self).__name__} with {type(other).__name__}"
+        raise NotImplementedError(msg)
+
+    def __neg__(self) -> Particle:
         return create_antiparticle(self)
 
     def is_lepton(self) -> bool:
@@ -210,29 +224,37 @@ class Particle:  # pylint: disable=too-many-instance-attributes
             p.text(f"{class_name}(...)")
         else:
             with p.group(indent=2, open=f"{class_name}("):
-                for field in attr.fields(type(self)):
-                    value = getattr(self, field.name)
-                    if value != field.default:
+                for attribute in attrs.fields(type(self)):  # type: ignore[misc]
+                    value = getattr(self, attribute.name)
+                    if value != attribute.default:
                         p.breakable()
-                        p.text(f"{field.name}=")
+                        p.text(f"{attribute.name}=")
                         if isinstance(value, Parity):
-                            p.text(_to_fraction(int(value), render_plus=True))
+                            p.text(_float_as_signed_str(int(value), render_plus=True))
                         else:
-                            p.pretty(value)
+                            p.pretty(value)  # type: ignore[attr-defined]
                         p.text(",")
             p.breakable()
             p.text(")")
 
 
-ParticleWithSpin = Tuple[Particle, float]
+def _get_name_root(name: str) -> str:
+    """Strip a string (particularly the `.Particle.name`) of specifications."""
+    name_root = name
+    name_root = re.sub(r"\(.+\)", "", name_root)
+    return re.sub(r"[\*\+\-~\d']", "", name_root)
 
 
-class ParticleCollection(abc.MutableSet):
+ParticleWithSpin = tuple[Particle, Fraction]
+"""A particle and its spin projection."""
+
+
+class ParticleCollection(abc.MutableSet):  # noqa: PLW1641
     """Searchable collection of immutable `.Particle` instances."""
 
-    def __init__(self, particles: Optional[Iterable[Particle]] = None) -> None:
-        self.__particles: Dict[str, Particle] = {}
-        self.__pid_to_name: Dict[int, str] = {}
+    def __init__(self, particles: Iterable[Particle] | None = None) -> None:
+        self.__particles: dict[str, Particle] = {}
+        self.__pid_to_name: dict[int, str] = {}
         if particles is not None:
             self.update(particles)
 
@@ -243,23 +265,19 @@ class ParticleCollection(abc.MutableSet):
             return instance in self.__particles.values()
         if isinstance(instance, int):
             return instance in self.__pid_to_name
-        raise NotImplementedError(
-            f"Cannot search for type {instance.__class__.__name__}"
-        )
+        msg = f"Cannot search for type {type(instance).__name__}"
+        raise NotImplementedError(msg)
 
     def __eq__(self, other: object) -> bool:
         if isinstance(other, abc.Iterable):
             return set(self) == set(other)
-        raise NotImplementedError(
-            f"Cannot compare {self.__class__.__name__} with  {self.__class__.__name__}"
-        )
+        msg = f"Cannot compare {type(self).__name__} with  {type(self).__name__}"
+        raise NotImplementedError(msg)
 
     def __getitem__(self, particle_name: str) -> Particle:
         if particle_name in self.__particles:
             return self.__particles[particle_name]
-        error_message = (
-            f"No particle with name '{particle_name}' in the database"
-        )
+        error_message = f"No particle with name '{particle_name}' in the database"
         candidates = [
             p.name
             for p in sorted(self, key=lambda p: p.mass)
@@ -279,19 +297,18 @@ class ParticleCollection(abc.MutableSet):
     def __len__(self) -> int:
         return len(self.__particles)
 
-    def __iadd__(
-        self, other: Union[Particle, "ParticleCollection"]
-    ) -> "ParticleCollection":
+    def __iadd__(self, other: Particle | ParticleCollection) -> Self:
         if isinstance(other, Particle):
             self.add(other)
         elif isinstance(other, ParticleCollection):
             self.update(other)
         else:
-            raise NotImplementedError(f"Cannot add {other.__class__.__name__}")
+            msg = f"Cannot add {type(other).__name__}"
+            raise NotImplementedError(msg)
         return self
 
     def __repr__(self) -> str:
-        output = f"{self.__class__.__name__}({{"
+        output = f"{type(self).__name__}({{"
         for particle in self:
             output += f"\n    {particle},"
         output += "})"
@@ -305,7 +322,7 @@ class ParticleCollection(abc.MutableSet):
             with p.group(indent=2, open=f"{class_name}({{"):
                 for particle in self:
                     p.breakable()
-                    p.pretty(particle)
+                    p.pretty(particle)  # type: ignore[attr-defined]
                     p.text(",")
             p.breakable()
             p.text("})")
@@ -314,49 +331,50 @@ class ParticleCollection(abc.MutableSet):
         if value in self.__particles.values():
             equivalent_particles = {p for p in self if p == value}
             equivalent_particle = next(iter(equivalent_particles))
+            msg = (
+                f'Added particle "{value.name}" is equivalent to existing particle'
+                f' "{equivalent_particle.name}"'
+            )
             raise ValueError(
-                f'Added particle "{value.name}" is equivalent to '
-                f'existing particle "{equivalent_particle.name}"',
+                (msg),
             )
         if value.name in self.__particles:
-            logging.warning(f'Overwriting particle with name "{value.name}"')
+            _LOGGER.warning(f'Overwriting particle with name "{value.name}"')
         if value.pid in self.__pid_to_name:
-            logging.warning(
-                f'Particle with PID {value.pid} already exists: "{self.find(value.pid).name}"'
+            _LOGGER.warning(
+                f"Particle with PID {value.pid} already exists:"
+                f' "{self.find(value.pid).name}"'
             )
         self.__particles[value.name] = value
         self.__pid_to_name[value.pid] = value.name
 
-    def discard(self, value: Union[Particle, str]) -> None:
+    def discard(self, value: Particle | str) -> None:
         particle_name = ""
         if isinstance(value, Particle):
             particle_name = value.name
         elif isinstance(value, str):
             particle_name = value
         else:
-            raise NotImplementedError(
-                f"Cannot discard something of type {value.__class__.__name__}"
-            )
+            msg = f"Cannot discard something of type {type(value).__name__}"
+            raise NotImplementedError(msg)
         del self.__pid_to_name[self[particle_name].pid]
         del self.__particles[particle_name]
 
-    def find(self, search_term: Union[int, str]) -> Particle:
+    def find(self, search_term: int | str) -> Particle:
         """Search for a particle by either name (`str`) or PID (`int`)."""
         if isinstance(search_term, str):
             particle_name = search_term
-            return self.__getitem__(particle_name)
+            return self[particle_name]
         if isinstance(search_term, int):
             if search_term not in self.__pid_to_name:
-                raise KeyError(f"No particle with PID {search_term}")
+                msg = f"No particle with PID {search_term}"
+                raise KeyError(msg)
             particle_name = self.__pid_to_name[search_term]
-            return self.__getitem__(particle_name)
-        raise NotImplementedError(
-            f"Cannot search for a search term of type {type(search_term)}"
-        )
+            return self[particle_name]
+        msg = f"Cannot search for a search term of type {type(search_term)}"
+        raise NotImplementedError(msg)
 
-    def filter(  # noqa: A003
-        self, function: Callable[[Particle], bool]
-    ) -> "ParticleCollection":
+    def filter(self, function: Callable[[Particle], bool]) -> ParticleCollection:
         """Search by `Particle` properties using a :code:`lambda` function.
 
         For example:
@@ -369,108 +387,91 @@ class ParticleCollection(abc.MutableSet):
         ...     and p.spin == 2
         ...     and p.strangeness == 1
         ... )
-        >>> sorted(list(subset.names))
-        ['K(2)(1820)+', 'K(2)(1820)0']
+        >>> sorted(subset.names)
+        ['K(2)(1820)+', 'K(2)(1820)0', 'K(2)*(1980)+', 'K(2)*(1980)0']
         """
-        return ParticleCollection(
-            {particle for particle in self if function(particle)}
-        )
+        return ParticleCollection({particle for particle in self if function(particle)})
 
     def update(self, other: Iterable[Particle]) -> None:
         if not isinstance(other, abc.Iterable):
-            raise TypeError(
-                f"Cannot update {self.__class__.__name__} from "
-                f"non-iterable class {self.__class__.__name__}"
+            msg = (
+                f"Cannot update {type(self).__name__} from non-iterable class"
+                f" {type(self).__name__}"
             )
+            raise TypeError(msg)
         for particle in other:
             self.add(particle)
 
     @property
-    def names(self) -> Set[str]:
-        return set(self.__particles)
+    def names(self) -> list[str]:
+        return [p.name for p in sorted(self)]
 
 
-def create_particle(  # pylint: disable=too-many-arguments,too-many-locals
+def create_particle(  # noqa: PLR0917
     template_particle: Particle,
-    name: Optional[str] = None,
-    latex: Optional[str] = None,
-    pid: Optional[int] = None,
-    mass: Optional[float] = None,
-    width: Optional[float] = None,
-    charge: Optional[int] = None,
-    spin: Optional[float] = None,
-    isospin: Optional[Spin] = None,
-    strangeness: Optional[int] = None,
-    charmness: Optional[int] = None,
-    bottomness: Optional[int] = None,
-    topness: Optional[int] = None,
-    baryon_number: Optional[int] = None,
-    electron_lepton_number: Optional[int] = None,
-    muon_lepton_number: Optional[int] = None,
-    tau_lepton_number: Optional[int] = None,
-    parity: Optional[int] = None,
-    c_parity: Optional[int] = None,
-    g_parity: Optional[int] = None,
+    name: str | None = None,
+    latex: str | None = None,
+    pid: int | None = None,
+    mass: float | None = None,
+    width: float | None = None,
+    charge: int | None = None,
+    spin: float | None = None,
+    isospin: Spin | None = None,
+    strangeness: int | None = None,
+    charmness: int | None = None,
+    bottomness: int | None = None,
+    topness: int | None = None,
+    baryon_number: int | None = None,
+    electron_lepton_number: int | None = None,
+    muon_lepton_number: int | None = None,
+    tau_lepton_number: int | None = None,
+    parity: int | None = None,
+    c_parity: int | None = None,
+    g_parity: int | None = None,
 ) -> Particle:
     return Particle(
-        name=name if name else template_particle.name,
-        pid=pid if pid else template_particle.pid,
-        latex=latex if latex else template_particle.latex,
+        name=name or template_particle.name,
+        pid=pid or template_particle.pid,
+        latex=latex or template_particle.latex,
         mass=mass if mass is not None else template_particle.mass,
-        width=width if width else template_particle.width,
-        spin=spin if spin else template_particle.spin,
-        charge=charge if charge else template_particle.charge,
-        strangeness=strangeness
-        if strangeness
-        else template_particle.strangeness,
-        charmness=charmness if charmness else template_particle.charmness,
-        bottomness=bottomness if bottomness else template_particle.bottomness,
-        topness=topness if topness else template_particle.topness,
-        baryon_number=baryon_number
-        if baryon_number
-        else template_particle.baryon_number,
-        electron_lepton_number=electron_lepton_number
-        if electron_lepton_number
-        else template_particle.electron_lepton_number,
-        muon_lepton_number=muon_lepton_number
-        if muon_lepton_number
-        else template_particle.muon_lepton_number,
-        tau_lepton_number=tau_lepton_number
-        if tau_lepton_number
-        else template_particle.tau_lepton_number,
-        isospin=template_particle.isospin
-        if isospin is None
-        else template_particle.isospin,
+        width=width or template_particle.width,
+        spin=spin or template_particle.spin,
+        charge=charge or template_particle.charge,
+        strangeness=strangeness or template_particle.strangeness,
+        charmness=charmness or template_particle.charmness,
+        bottomness=bottomness or template_particle.bottomness,
+        topness=topness or template_particle.topness,
+        baryon_number=(baryon_number or template_particle.baryon_number),
+        electron_lepton_number=(
+            electron_lepton_number or template_particle.electron_lepton_number
+        ),
+        muon_lepton_number=(muon_lepton_number or template_particle.muon_lepton_number),
+        tau_lepton_number=(tau_lepton_number or template_particle.tau_lepton_number),
+        isospin=template_particle.isospin if isospin is None else isospin,
         parity=template_particle.parity if parity is None else Parity(parity),
-        c_parity=template_particle.c_parity
-        if c_parity is None
-        else Parity(c_parity),
-        g_parity=template_particle.g_parity
-        if g_parity is None
-        else Parity(g_parity),
+        c_parity=template_particle.c_parity if c_parity is None else Parity(c_parity),
+        g_parity=template_particle.g_parity if g_parity is None else Parity(g_parity),
     )
 
 
 def create_antiparticle(
     template_particle: Particle,
-    new_name: Optional[str] = None,
-    new_latex: Optional[str] = None,
+    new_name: str | None = None,
+    new_latex: str | None = None,
 ) -> Particle:
-    isospin: Optional[Spin] = None
+    isospin: Spin | None = None
     if template_particle.isospin:
         isospin = -template_particle.isospin
-    parity: Optional[Parity] = None
+    parity: Parity | None = None
     if template_particle.parity is not None:
-        if template_particle.spin.is_integer():
+        if template_particle.spin.denominator == 1:
             parity = template_particle.parity
         else:
             parity = -template_particle.parity
     return Particle(
-        name=new_name if new_name else "anti-" + template_particle.name,
+        name=new_name or "anti-" + template_particle.name,
         pid=-template_particle.pid,
-        latex=new_latex
-        if new_latex
-        else fR"\overline{{{template_particle.latex}}}",
+        latex=new_latex or Rf"\overline{{{template_particle.latex}}}",
         mass=template_particle.mass,
         width=template_particle.width,
         charge=-template_particle.charge,
@@ -496,6 +497,8 @@ def load_pdg() -> ParticleCollection:
     PDG info is imported from the `scikit-hep/particle
     <https://github.com/scikit-hep/particle>`_ package.
     """
+    from particle import Particle as PdgDatabase  # noqa: PLC0415
+
     all_pdg_particles = PdgDatabase.findall(
         lambda item: item.charge is not None
         and item.charge.is_integer()  # remove quarks
@@ -519,23 +522,24 @@ __skip_particles = {
 }
 
 
-def __sign(value: Union[float, int]) -> int:
+def __sign(value: float) -> int:
     return int(copysign(1, value))
 
 
 # cspell:ignore pdgid
 def __convert_pdg_instance(pdg_particle: PdgDatabase) -> Particle:
-    def convert_mass_width(value: Optional[float]) -> float:
+    def convert_mass_width(value: float | None) -> float:
         if value is None:
             return 0.0
         return float(value) / 1e3  # https://github.com/ComPWA/qrules/issues/14
 
     if pdg_particle.charge is None:
-        raise ValueError(f"PDG instance has no charge:\n{pdg_particle}")
+        msg = f"PDG instance has no charge:\n{pdg_particle}"
+        raise ValueError(msg)
     quark_numbers = __compute_quark_numbers(pdg_particle)
     lepton_numbers = __compute_lepton_numbers(pdg_particle)
     if pdg_particle.pdgid.is_lepton:  # convention: C(fermion)=+1
-        parity: Optional[Parity] = Parity(__sign(pdg_particle.pdgid))  # type: ignore
+        parity: Parity | None = Parity(__sign(pdg_particle.pdgid))
     else:
         parity = __create_parity(pdg_particle.P)
     latex = None
@@ -566,7 +570,7 @@ def __convert_pdg_instance(pdg_particle: PdgDatabase) -> Particle:
 
 def __compute_quark_numbers(
     pdg_particle: PdgDatabase,
-) -> Tuple[int, int, int, int]:
+) -> tuple[int, int, int, int]:
     strangeness = 0
     charmness = 0
     bottomness = 0
@@ -587,7 +591,7 @@ def __compute_quark_numbers(
 
 def __compute_lepton_numbers(
     pdg_particle: PdgDatabase,
-) -> Tuple[int, int, int]:
+) -> tuple[int, int, int]:
     electron_lepton_number = 0
     muon_lepton_number = 0
     tau_lepton_number = 0
@@ -606,17 +610,18 @@ def __compute_baryonnumber(pdg_particle: PdgDatabase) -> int:
     return int(__sign(pdg_particle.pdgid) * pdg_particle.pdgid.is_baryon)
 
 
-def __create_isospin(pdg_particle: PdgDatabase) -> Optional[Spin]:
+def __create_isospin(pdg_particle: PdgDatabase) -> Spin | None:
     if pdg_particle.I is None:
         return None
-    magnitude = pdg_particle.I
+    magnitude = Fraction(pdg_particle.I)
     projection = __isospin_projection_from_pdg(pdg_particle)
     return Spin(magnitude, projection)
 
 
-def __isospin_projection_from_pdg(pdg_particle: PdgDatabase) -> float:
+def __isospin_projection_from_pdg(pdg_particle: PdgDatabase) -> Fraction:
     if pdg_particle.charge is None:
-        raise ValueError(f"PDG instance has no charge:\n{pdg_particle}")
+        msg = f"PDG instance has no charge:\n{pdg_particle}"
+        raise ValueError(msg)
     if "qq" in pdg_particle.quarks.lower():
         strangeness, charmness, bottomness, topness = __compute_quark_numbers(
             pdg_particle
@@ -632,12 +637,10 @@ def __isospin_projection_from_pdg(pdg_particle: PdgDatabase) -> float:
             projection += quark_content.count("u") + quark_content.count("D")
             projection -= quark_content.count("U") + quark_content.count("d")
             projection *= 0.5
-    if (
-        pdg_particle.I is not None
-        and not (pdg_particle.I - projection).is_integer()
-    ):
-        raise ValueError(f"Cannot have isospin {(pdg_particle.I, projection)}")
-    return projection
+    if pdg_particle.I is not None and not (pdg_particle.I - projection).is_integer():
+        msg = f"Cannot have isospin {pdg_particle.I, projection}"
+        raise ValueError(msg)
+    return Fraction(projection)
 
 
 def __filter_quark_content(pdg_particle: PdgDatabase) -> str:
@@ -647,7 +650,9 @@ def __filter_quark_content(pdg_particle: PdgDatabase) -> str:
     return matches[1]
 
 
-def __create_parity(parity_enum: enums.Parity) -> Optional[Parity]:
+def __create_parity(parity_enum: enums.Parity) -> Parity | None:
+    from particle.particle import enums  # noqa: PLC0415
+
     if parity_enum is None or parity_enum == enums.Parity.u:
         return None
     if parity_enum == getattr(parity_enum, "o", None):  # particle < 0.14
