@@ -1,8 +1,8 @@
-import qrules as q
+import qrules
 
 
 def test_script():
-    result = q.generate_transitions(
+    reaction = qrules.generate_transitions(
         initial_state="D0",
         final_state=["K~0", "K+", "K-"],
         allowed_intermediate_particles=[
@@ -10,13 +10,16 @@ def test_script():
             "a(2)(1320)-",
             "phi(1020)",
         ],
-        number_of_threads=1,
     )
-    assert len(result.transitions) == 5
-    assert result.get_intermediate_particles().names == {
-        "a(0)(980)+",
+    groupings = sorted(reaction.group_by_topology().values())
+    assert len(groupings) == 3
+    assert len(groupings[0]) == 2
+    assert len(groupings[1]) == 2
+    assert len(groupings[2]) == 1
+    assert reaction.get_intermediate_particles().names == [
         "a(0)(980)-",
         "a(0)(980)0",
+        "a(0)(980)+",
         "a(2)(1320)-",
         "phi(1020)",
-    }
+    ]
