@@ -8,4 +8,4 @@ from qrules.conservation_rules import ChargeConservation
     [(([0], [-1, 1]), True), (([0], [1, 1]), False)],
 )
 def test_charge_conservation(graph_input, expected_value):
-    assert ChargeConservation()(*graph_input) == expected_value
+    assert ChargeConservation()(*graph_input) == expected_value  # type: ignore[abstract]
