@@ -1,4 +1,3 @@
-# pylint: disable=redefined-outer-name
 import particle
 import pytest
 
@@ -32,14 +31,22 @@ def test_maybe_qq():
 
 
 def test_pdg_size(pdg: ParticleCollection):
-    assert len(pdg) in [
+    assert len(pdg) in {
         512,  # particle==0.13
-        519,  # particle==0.14
-    ]
-    assert len(pdg.filter(lambda p: "~" in p.name)) in [
+        519,  # particle==0.14-0.15
+        531,  # particle==0.16-0.20
+        530,  # particle==0.21-0.22
+        537,  # particle==0.23-0.24
+        539,  # particle==0.25
+        540,  # particle==0.25.4
+        541,  # particle==0.26.0
+    }
+    assert len(pdg.filter(lambda p: "~" in p.name)) in {
         165,  # particle==0.13
-        172,  # particle==0.14
-    ]
+        172,  # particle==0.14, 0.15
+        175,  # particle==0.16-0.24
+        176,  # particle==0.25
+    }
 
 
 def test_missing_in_pdg(
